#!/usr/bin/perl

#######################################################################################
#
# Check HPE MSA P2000
#
#
# v1.0 11/2018 JH
#
# checks a HP P2000 MSA Storage
#
# vdisks, Power-Supplies, Fans, SAS, disks, Management Network Ports
#
#
# based on: 
#
# HPE Documentation 
# check_hp_msa2000.pl from Marco Klopfer (www.kingmarco.de)
# try & error
#
#
#
#
#########################################################################################

use LWP::UserAgent;
use Digest::MD5 qw(md5_hex);
use XML::LibXML;
use Getopt::Long;
use Switch;
use Array::Utils qw(:all);
use Net::Ping;


sub print_help ();
sub print_usage ();

#------------- Parameter ---------------------------#

Getopt::Long::Configure('bundling');
GetOptions
    ("v"   => \$opt_v, "version"    => \$opt_V,
     "h"   => \$opt_h, "help"       => \$opt_h,
     "u=s" => \$opt_u, "username=s" => \$opt_u,
     "p=s" => \$opt_p, "password=s" => \$opt_p,
     "d=s" => \$opt_d, "debug=s"  => \$opt_d,
     "S=s" => \$opt_S, "sparecnt" => \$opt_S,
     "H=s" => \$opt_H, "hostname=s" => \$opt_H);

$PROGNAME ="p2000_http_login.pl";
$endebug=1;
$myDebug="0";

#
#
# ====================  Configuration ===================== #
#
# Disks / Enclosure
#
# Array @mydisks = ('1.1','1.2','1.3'....'2,12' ) <'Enclosure ID,Disk'>
#
@myDisks = ('1.1','1.2','1.3','1.4','1.5','1.6','1.7','1.8','1.9','1.10','1.11','1.12','2.1','2.2','2.3','2.4','2.5','2.6','2.7','2.8','2.9','2.10','2.11','2.12');

# 
# amount Spare Disks
#
$mySpareDisks = "2";

#
# secondary controller = Parameter: -S <-S10.154.3.1>
#

# ========================================================= #



#
# -------- check submitted parameter ---------
#

if ($opt_v) {
    print "Version 1.0 11/2018 \n";
    exit 0;
}
if ($opt_h) {
    print_help();
    exit 0;
}

$opt_H = shift unless ($opt_H);
my $Chost = $1 if  ($opt_H && $opt_H =~ m/^([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+|[-a-zA-Z][-a-zA-z0-9]+(\.[a-zA-Z][-a-zA-Z0-9]+)*)$/);
unless ($Chost) {
	print "No Host defined \n";
	print_usage();
	exit 3;
}
#print "Hostname: $host  ";

$opt_S = shift unless ($opt_S);
my $Shost = $1 if  ($opt_S && $opt_S =~ m/^([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+|[-a-zA-Z][-a-zA-z0-9]+(\.[a-zA-Z][-a-zA-Z0-9]+)*)$/);

if ($Shost){
 #print "SpareController: $Shost \n";
 $mySecondContr = $Shost;
}

$opt_u = shift unless ($opt_u);
my $username = $2 if ($opt_u && $opt_u =~ m/^(([-a-zA-Z0-9]+)*)$/);
unless ($username) {
	print "No Username defined \n";
	print_usage();
	exit 3;
}
#print "Username: $username   ";


$opt_p = shift unless ($opt_p);
my $password = $2 if ($opt_p && $opt_p =~ m/^(([-a-zA-Z0-9\!]+)*)$/);
unless ($password) {
	print "No Password defined \n";
	print_usage();
	exit 3;
}
#print "Password: $password \n";

$opt_d = shift unless ($opt_d);
my $endebug = $1 if ($opt_d && $opt_d =~ m/^(([0-2]+))$/);

if ( $endebug < "1" ) {
	# print "Debug OFF";
	$myDebug="0";
    }else {
	# print "Debug ON";
	$myDebug="1";
}



#  create MD5-Hash from username & password
#
#
my $md5_data = $username."_".$password;

#my $md5_data = "manage_!manage";
my $md5_hash = md5_hex( $md5_data );



#
# =====> Is Controller available ? <=======
#

if ($Shost){
    @cHosts = ("$Chost","$mySecondContr");
 } else {
    @cHosts = ("$Chost");
    $mySecondContr = ""; 
}

$c2_onl="0";
$c1_onl="0";

my @cStatus;
$i = "0";

foreach $Ctrl (@cHosts) {
 $cStat =  ping($Ctrl);
 $cStatus[$i][0] = $Ctrl;
 $cStatus[$i][1] = $cStat;
 
 if ( $Ctrl eq $Chost && $cStat eq "0" ) {
    $c1_onl="1";
    $host = $Ctrl;
 } 
 if ( $Ctrl eq $mySecondContr && $cStat eq "0" && $c1_onl eq "0" ) {
    $c2_onl="1";
    $host = $mySecondContr;
 } 
 if ( $Ctrl eq $mySecondContr && $cStat eq "2" ) {
    $c2_onl="0";
 } 
 $i++;
}

if ( $c1_onl eq "0" && $c2_onl eq "0" ) {
    print "CRITICAL: Can't connect to Controller: $Chost $mySecondContr \n";
    exit 2;
}

#
# -------- Login into Array ----------------
#

# create an URL and send a  HTTP-GET-Request
$ua = LWP::UserAgent->new;

#$url = 'http://10.154.2.1/api/login/' . $md5_hash;
$url = 'http://' . $host . '/api/login/' . $md5_hash;

$req = HTTP::Request->new(GET => $url);
$res = $ua->request($req);
#print $res->content;

my $parser = XML::LibXML->new();
my $doc = $parser->parse_string( $res->content );
my $root = $doc->getDocumentElement;
my @objects = $root->getElementsByTagName('OBJECT');
my @props = $objects[0]->getElementsByTagName('PROPERTY');
my $sessionKey;
foreach my $prop ( @props ) {
    my $name = $prop->getAttribute('name');
    # print "Property = " . $name . "\n";
    if( $name eq 'return-code' ) {
	$returncode = $prop->textContent;
    }
    if( $name eq 'response' ) {
	$sessionKey = $prop->textContent;
    }
}
#-------- check login ---------

if ( $returncode == "2" ) {
    print "ERROR: Login failed \n";
    exit 3;
}

#
# ---- get system information --------
#
#
$url = 'http://' . $host . '/api/show/system';
$req = HTTP::Request->new(GET => $url);
$req->header('sessionKey' => $sessionKey );
$req->header('dataType' => 'ipa' );
$res = $ua->request($req);

## Parse XML

 $parser = XML::LibXML->new();
 $doc = $parser->parse_string( $res->content );
 $root = $doc->getDocumentElement;
 @objects = $root->getElementsByTagName('OBJECT');
$i = 0 ;
$storage_counter = 0;

### Loop for XML tag GROUPS
     foreach my $objects_count (@objects) {
    @props = $objects[$i]->getElementsByTagName('PROPERTY');
     if ( $objects_count->getAttribute('basetype') eq "system" ) {
$sessionKey;
### Loop for XML tag PROPERTYS
        foreach my $prop ( @props ) {
        my $name = $prop->getAttribute('name');
         if( $name eq 'system-name' ) { $storage_state{'name'}[$i] = $prop->textContent };
         if( $name eq 'system-contact' ) { $storage_state{'contact'}[$i] = $prop->textContent };
         if( $name eq 'system-location' ) { $storage_state{'location'}[$i] = $prop->textContent };
         if( $name eq 'vendor-name' ) { $storage_state{'vendor'}[$i] = $prop->textContent };
         if( $name eq 'product-id' ) { $storage_state{'id'}[$i] = $prop->textContent };
         if( $name eq 'product-brand' ) { $storage_state{'brand'}[$i] = $prop->textContent };
                    $ii++;
                                    }
     $storage_counter++;
                         } else {
### Check Response OK
         foreach my $prop ( @props ) {
         my $name = $prop->getAttribute('name');
          if( $name eq 'response-type' && $prop->textContent eq "Success" ) { $response_success = 1 }
                                     }
 }

 $i++;
}

### 
#
#
#
# ----------------------------- show vdisks ------------------------------------------
#
#$url = 'http://10.154.2.1/api/show/ports';
$url = 'http://' . $host . '/api/show/vdisks';
$req = HTTP::Request->new(GET => $url);
$req->header('sessionKey' => $sessionKey );
$req->header('dataType' => 'ipa' );
$res = $ua->request($req);

## Parse XML

 $parser = XML::LibXML->new();
 $doc = $parser->parse_string( $res->content );
 $root = $doc->getDocumentElement;
 @objects = $root->getElementsByTagName('OBJECT');
$i = 0 ;
### Loop for XML tag GROUPS
     foreach my $objects_count (@objects) {
    @props = $objects[$i]->getElementsByTagName('PROPERTY');
     if ( $objects_count->getAttribute('basetype') eq "virtual-disks" ) {
$sessionKey;

### ------ Loop for XML tag PROPERTYS -----------

        foreach my $prop ( @props ) {
        my $name = $prop->getAttribute('name');
         if( $name eq 'name' ) { $disks_state{'name'}[$i] = $prop->textContent };
         if( $name eq 'status' ) { $disks_state{'state'}[$i] = $prop->textContent };
         if( $name eq 'serial-number' ) { $disks_state{'serial'}[$i] = $prop->textContent };
         if( $name eq 'size' ) { $disks_state{'size'}[$i] = $prop->textContent };
         if( $name eq 'freespace' ) { $disks_state{'freespace'}[$i] = $prop->textContent };
         if( $name eq 'raidtype' ) { $disks_state{'raidtype'}[$i] = $prop->textContent };
         if( $name eq 'job-running' ) { $disks_state{'job-running'}[$i] = $prop->textContent };
         if( $name eq 'current-job-completion' ) { $disks_state{'job-complete'}[$i] = $prop->textContent };
                    $ii++;
                                    }
     $disk_counter++;
                         } else {

### ------- Check Response OK -----------------

         foreach my $prop ( @props ) {
          my $name = $prop->getAttribute('name');
          if( $name eq 'response-type' && $prop->textContent eq "Success" ) { $response_success = 1 }
        }                                     
 ;}

$i++;
}

### check vdisk status

# if job-running = RCON -> print job-complete

$i = 0;
$exit1_message = "";
$exit_status = 0;

do {
 switch ($disks_state{'state'}[$i]) {
  case("FTOL") { 
    if ( $myDebug eq "1" ){
	    $exit1_message .= "vdisk: ". $disks_state{'name'}[$i]." is online. ";
	} else {
	    $exit1_message .= "";
    }
  }
  case("CRIT"){ 
    if ( $exit_status lt 2){ 
	$exit_status = 2; 
	$exit1_message .= "vdisk: $disks_state{'name'}[$i] not fault tolerant, ";
	if ( $disks_state{'job-running'}[$i] eq "RCON" ){
	    $exit1_message .= "Resync progress:". $disks_state{'job-complete'}[$i]." ";
	}
    }
  }
  case("OFFL") { { if ( $exit_status lt 2) {$exit_status = 2 } ; $exit1_message .= "vdisk: $disks_state{'name'}[$i] offline and failed. "} }
  case("QTCR") { { if ( $exit_status lt 2) {$exit_status = 2 } ; $exit1_message .= "vdisk: $disks_state{'name'}[$i] has been quarantined - missing disks. "} }
  case("QTOF") { { if ( $exit_status lt 2) {$exit_status = 2 } ; $exit1_message .= "vdisk: $disks_state{'name'}[$i] has been quarantined - missing disks. "} }
  case("UP") { { if ( $exit_status lt 1) {$exit_status = 1 } ; $exit1_message .= "vdisk: $disks_state{'name'}[$i] not fault tolerant"} }
else { if ( $exit_status lt 2 ){$exit_status = 2 } ; $exit1_message .= "vdisk: $disks_state{'name'}[$i] unknown state: $disks_state{'state'}[$i]. "}
}
$i++; } while ( $i < $ disk_counter );

$vd_exit_status = $exit_status;

#
#
#
# ------------------------------ show power-supplies ------------------------------------------#
#
#
$url = 'http://' . $host . '/api/show/power-supplies';
$req = HTTP::Request->new(GET => $url);
$req->header('sessionKey' => $sessionKey );
$req->header('dataType' => 'ipa' );
$res = $ua->request($req);

## Parse XML

 $parser = XML::LibXML->new();
 $doc = $parser->parse_string( $res->content );
 $root = $doc->getDocumentElement;
 @objects = $root->getElementsByTagName('OBJECT');
$i = 0 ;
$ps_counter = 0 ;
$my_psc = 0;
$my_fanc = 0;

### Loop for XML tag GROUPS
     foreach my $objects_count (@objects) {
    @props = $objects[$i]->getElementsByTagName('PROPERTY');
     if ( $objects_count->getAttribute('basetype') eq "power-supplies" ) {
$sessionKey;
### Loop for XML tag PROPERTYS
        foreach my $prop ( @props ) {
          my $name = $prop->getAttribute('name');
          my $subname = substr $prop->textContent, 0, 3;
        
          # --- PSU
          #
          if( $name eq 'name' && $subname eq 'PSU' ) {
            $ps_state{'name'}[$my_psc] = $prop->textContent;
            for ( $j=0; $j <= 30; $j++ ) {
        	$prop = $props[$j];
		my $name = $prop->getAttribute('name');
        	if( $name eq 'status' ) { $ps_state{'state'}[$my_psc] = $prop->textContent };
        	if( $name eq 'serial-number' ) { $ps_state{'serial'}[$my_psc] = $prop->textContent };
        	if( $name eq 'location' ) { $ps_state{'location'}[$my_psc] = $prop->textContent };
        	if( $name eq 'health' ) { $ps_state{'health'}[$my_psc] = $prop->textContent };
        	if( $name eq 'enclosure-id' ) { $ps_state{'enc_id'}[$my_psc] = $prop->textContent };
            }
	    $my_psc++;
	  }
	  
          # --- FAN
          #
           $k_end = $j +'14';
           
	  if( $name eq 'durable-id' && $subname eq 'fan' ) {
            $fan_state{'name'}[$my_fanc] = $prop->textContent;        
            for ( $k=$j; $k <= $k_end; $k++ ) {
        	$prop = $props[$k];
		my $name = $prop->getAttribute('name');				
        	if( $name eq 'status' ) { $fan_state{'state'}[$my_fanc] = $prop->textContent };
        	if( $name eq 'location' ) { $fan_state{'location'}[$my_fanc] = $prop->textContent };
        	if( $name eq 'health' ) { $fan_state{'health'}[$my_fanc] = $prop->textContent };
        	if( $name eq 'durable-id' ) { $fan_state{'du_id'}[$my_fanc] = $prop->textContent };
            }
	    $my_fanc++;
	  }
          $ii++; 
        }
    	$ps_counter++;
    } else {
	 ### Check Response OK
	 foreach my $prop ( @props ) {
         my $name = $prop->getAttribute('name');
          if( $name eq 'response-type' && $prop->textContent eq "Success" ) { $response_success = 1 }
                                     }
    ;}
$i++;
}

### check power-supplies status

$i = 0;
$exit2_message = "";
$exit_status = 0;

do {
    switch ($ps_state{'health'}[$i]) {
	case("OK") { $exit2_message .= "Enclosure $ps_state{'enc_id'}[$i] $ps_state{'name'}[$i] OK. "}
	case("Fault") { { if ( $exit_status lt 2) {$exit_status = 2 } ; $exit2_message .= "Enclosure $ps_state{'enc_id'}[$i] $ps_state{'name'}[$i] FAILED. "} }
	case("Not Present") { { if ( $exit_status lt 2) {$exit_status = 2 } ; $exit2_message .= "Enclosure $ps_state{'enc_id'}[$i] $ps_state{'name'}[$i] nor present. "} }
    else { if ( $exit_status lt 2 ){$exit_status = 2 } ; $exit2_message .= "$ps_state{'name'}[$i] unknown state: $ps_state{'state'}[$i]. "}
    }
$i++; } while ( $i < $ps_counter );

$ps_exit_status = $exit_status;


### check PS-Fan status

$i = 0;
$exit3_message = "";
$exit_status = 0;

do {
    switch ($fan_state{'health'}[$i]) {
	case("OK") { $exit3_message .= "$fan_state{'location'}[$i] $fan_state{'du_id'}[$i] is OK. "}
    else { if ( $exit_status lt 2 ){$exit_status = 2 } ; $exit3_message .= "$fan_state{'name'}[$i] unknown state: $fan_state{'state'}[$i]. "}
    }
$i++; } while ( $i < $ps_counter );

$fan_exit_status = $exit_status;


#
#
#
# ------------------------------ show sas ------------------------------------------#
#
#
$url = 'http://' . $host . '/api/show/sas';
$req = HTTP::Request->new(GET => $url);
$req->header('sessionKey' => $sessionKey );
$req->header('dataType' => 'ipa' );
$res = $ua->request($req);

## Parse XML

 $parser = XML::LibXML->new();
 $doc = $parser->parse_string( $res->content );
 $root = $doc->getDocumentElement;
 @objects = $root->getElementsByTagName('OBJECT');
$i = 0 ;
### Loop for XML tag GROUPS
foreach my $objects_count (@objects) {
    @props = $objects[$i]->getElementsByTagName('PROPERTY');
     if ( $objects_count->getAttribute('basetype') eq "expander-ports" ) {
$sessionKey;
### Loop for XML tag PROPERTYS
        foreach my $prop ( @props ) {
            my $name = $prop->getAttribute('name');
            if( $name eq 'name' ) { $sas_state{'name'}[$i] = $prop->textContent };
            if( $name eq 'health' ) { $sas_state{'health'}[$i] = $prop->textContent };
            if( $name eq 'durable-id' ) { $sas_state{'du_id'}[$i] = $prop->textContent };
            if( $name eq 'enclosure-id' ) { $sas_state{'enc_id'}[$i] = $prop->textContent };
            $ii++;
         }
         $sas_counter++;
    } else {
### Check Response OK
         foreach my $prop ( @props ) {
        	my $name = $prop->getAttribute('name');
        	if( $name eq 'response-type' && $prop->textContent eq "Success" ) { $response_success = 1 }
         }
                                     
   ;}

    $i++;
}

### check sas status

$i = 0;
$exit4_message = "";
$exit_status = 0;

do {
    switch ($sas_state{'health'}[$i]) {
	case("OK") { $exit4_message .= "Enclosure $sas_state{'enc_id'}[$i] $sas_state{'du_id'}[$i] is OK. "}
	case("N/A") { $exit4_message .= "Enclosure $sas_state{'enc_id'}[$i] $sas_state{'du_id'}[$i] not connected. "}
	case("Fault") { { if ( $exit_status lt 2) {$exit_status = 2 } ; $exit4_message .= "Enclosure $sas_state{'enc_id'}[$i] $sas_state{'du_id'}[$i] FAULT: Port is inoperable. "} }
    else { if ( $exit_status lt 2 ){$exit_status = 2 } ; $exit4_message .= "Enclosure $sas_state{'enc_id'}[$i] $sas_state{'du_id'}[$i] unknown state: $disks_state{'health'}[$i]. "}
    }
$i++; } while ( $i < $ sas_counter );

$sas_exit_status = $exit_status;

#
#
#
# ------------------------------ show disks ------------------------------------------#
#
#
$url = 'http://' . $host . '/api/show/disks';
$req = HTTP::Request->new(GET => $url);
$req->header('sessionKey' => $sessionKey );
$req->header('dataType' => 'ipa' );
$res = $ua->request($req);

## Parse XML

 $parser = XML::LibXML->new();
 $doc = $parser->parse_string( $res->content );
 $root = $doc->getDocumentElement;
 @objects = $root->getElementsByTagName('OBJECT');
$i = 0 ;
$hdd_counter = 0;
@myonlhdd = "";
$mySpareOnline = 0;

### Loop for XML tag GROUPS
foreach my $objects_count (@objects) {
    @props = $objects[$i]->getElementsByTagName('PROPERTY');
     if ( $objects_count->getAttribute('basetype') eq "drives" ) {
$sessionKey;

# print $res->content;

### Loop for XML tag PROPERTYS
        foreach my $prop ( @props ) {
            my $name = $prop->getAttribute('name');
            if( $name eq 'slot' ) { $hdd_state{'slot'}[$i] = $prop->textContent };
            if( $name eq 'health' ) { $hdd_state{'health'}[$i] = $prop->textContent };
            if( $name eq 'durable-id' ){ $hdd_state{'du_id'}[$i] = $prop->textContent };
            if( $name eq 'state' ) { $hdd_state{'state'}[$i] = $prop->textContent };
            if ( $name eq 'location' ){
        	$myonlhdd[$i] = $prop->textContent;
        	# print "HDD: --$myonlhdd[$i]-- ";
	    }
            if( $name eq 'enclosure-id' ) { $hdd_state{'enc_id'}[$i] = $prop->textContent };
            $ii++;
         }
         $hdd_counter++;
    } else {
        ### ----- Check Response OK ------
         foreach my $prop ( @props ) {
        	my $name = $prop->getAttribute('name');
        	if( $name eq 'response-type' && $prop->textContent eq "Success" ) { $response_success = 1 }
         }
   ;}

    $i++;
}

### check drives status


# --- how many drives are installed ? ---

$my_hdd_cnt = @myDisks;

# --- how many drives are present ? ---

$my_hdd_onl = @myonlhdd;

$i = 0;
$exit5_message = "";
$exit_status = 0;
$my_missing = "0";

# --- compare disks online vs disks defined  ---

if ( $my_hdd_onl ne $my_hdd_cnt ) {
    my @diff = array_minus(@myDisks, @myonlhdd );
    $misslist = join(", ", @diff);
    $my_missing = "1";
}

# ----- disk health checks --------
do {
 switch ($hdd_state{'health'}[$i]) {
 case("OK"){
    if ( $myDebug eq "1" ){
	    $exit5_message .= "Enclosure $hdd_state{'enc_id'}[$i] $hdd_state{'du_id'}[$i] is OK. ";
	} else {
	    $exit5_message .= "";
    }
 }
 case("N/A") { $exit5_message .= "Enclosure $hdd_state{'enc_id'}[$i] $hdd_state{'du_id'}[$i] not connected. "}
 case("Fault") { { if ( $exit_status lt 2) {$exit_status = 2 } ; $exit5_message .= "Enclosure $hdd_state{'enc_id'}[$i] $hdd_state{'du_id'}[$i] FAULT: Port is inoperable. "} }
    else { if ( $exit_status lt 2 ){$exit_status = 2 } ; $exit5_message .= "Enclosure $hdd_state{'enc_id'}[$i] $hdd_state{'du_id'}[$i] unknown state: $hdd_state{'health'}[$i]. "}
 }

 # -- count Spare Disks ---
 #
    if ( $hdd_state{'state'}[$i] eq "VDISK SP" ) {
	$mySpareOnline++;
    }

$i++; } while ( $i < $ hdd_counter );

# --- all Spares defined and online ? ---

if ( $mySpareOnline ne $mySpareDisks ) {
    $exit5_message .= " => SpareDisk counter mismatch Online: $mySpareOnline expected: $mySpareDisks";
    $exit_status = 1;
}


# --- all Disks Online ? ---

if ( $my_missing == "1" && $exit_status lt 2 ) {
    $exit_status = 2;
    $exit5_message .= " => HDD missing: $misslist";
}

$hdd_exit_status = $exit_status;


# --- Management Network status ----

$i = 0;
$exit6_message = "";
$nw_exit_status = "0";
$mgt_int = "";

foreach $myNWStat (@cHosts) {
    if ($cStatus[$i][1] eq "0"){ $mgt_int .= "Controller: ".$cStatus[$i][0]." OK \n" };
    if ($cStatus[$i][1] ne "0"){
	$exit6_message .= "Controller: ".$cStatus[$i][0]. " OFFLINE \n";
	$mgt_int .= $exit6_message;
	if ( $nw_exit_status lt 2) {$nw_exit_status = 1 };
    }
    $i++;
}


# ------------------------------ Logout ------------------------------------------#

$url2 = 'http://10.154.2.1/api/exit';
$req2 = HTTP::Request->new(GET => $url3);
$req2->header('sessionKey' => $sessionKey );
$req2->header('dataType' => 'api' );
$res2 = $ua->request($req2);

# print $res->content;



# --------- get system info -----------------------
#
#
$mydevice = $storage_state{'name'}[0] . " ";
# print $storage_state{'contact'}[0] . "\n";
# print $storage_state{'location'}[0] . "\n";
$mydevice .= $storage_state{'vendor'}[0] . " ";
$mydevice .= $storage_state{'id'}[0] . " ";
$mydevice .= $storage_state{'brand'}[0] . " ";

#
# ---------- Debugmode on / off -------------------

if ( $myDebug eq "1" ){
	print "\n ==> Device: $mydevice || Contact: $storage_state{'contact'}[0] || Location: $storage_state{'location'}[0]  <==\n";
	print "\nDebugmode ON \n\n";
	print "vdisks: \n" . $exit1_message . " Status: " . $vd_exit_status ."\n\n";
	print "PS:     \n" . $exit2_message . " Status: " . $ps_exit_status ."\n\n";
	print "FAN:    \n" . $exit3_message . " Status: " . $fan_exit_status ."\n\n";
	print "SAS:    \n" . $exit4_message . " Status: " . $sas_exit_status ."\n\n";
	print "HDD:    \n" . $exit5_message . " Status: " . $hdd_exit_status ."\n\n";
	print "VDISK SP online: \n" . $mySpareOnline . " Disks Online \n\n";
	print "Management Interfaces: \n" . $mgt_int ." \n\n";
}

#
# Check if any error exists --> highest errorlevel will be used.
#
$myError = "";
$exitstate = "0";

if ( $vd_exit_status eq "0" ) {
	$exitstate = "0";
    } else {
	if ( $vd_exit_status gt $exitstate ) {
	    $exitstate = $vd_exit_status;	
	}
	$myError .= $exit1_message;
}

if ( $ps_exit_status eq "0" ) {
	$exitstate = $exitstate;
    } else {
	if ( $ps_exit_status gt $exitstate ) {
	    $exitstate = $ps_exit_status;	
	}
	$myError .= $exit2_message;
}	

if ( $fan_exit_status eq "0" ) {
	$exitstate = $exitstate;
    } else {
	if ( $fan_exit_status gt $exitstate ) {
	    $exitstate = $fan_exit_status;	
	}
	$myError .= $exit3_message;
}
	
if ( $sas_exit_status eq "0" ) {
	$exitstate = $exitstate;
    } else {
	if ( $sas_exit_status gt $exitstate ) {
	    $exitstate = $sas_exit_status;	
	}
	$myError .= $exit4_message;
}
	
if ( $hdd_exit_status eq "0" ) {
	$exitstate = $exitstate;
    } else {
	if ( $hdd_exit_status gt $exitstate ) {
	    $exitstate = $hdd_exit_status;	
	}
	$myError .= $exit5_message;
}	

if ( $nw_exit_status eq "0" ) {
	$exitstate = $exitstate;
    } else {
	if ( $nw_exit_status gt $exitstate ) {
	    $exitstate = $nw_exit_status;	
	}
	$myError .= $exit6_message;
}	


#
#
# Generate output & errorlevel
#
#

switch($exitstate) {
    case("0") {
	print "Systemstatus: $mydevice OK \n";
	exit 0;
    }
    case("1") {
	print "WARNING: $storage_state{'name'}[0]  ".$myError."\n";
	exit 1;
    }
    case("2") {
	print "CRITICAL: $storage_state{'name'}[0]  ".$myError."\n";
	exit 2;
    }
    case("3"){
	print "UNKNOWN: $storage_state{'name'}[0]  ".$myError."\n";
	exit 3;
    }

}

#
#
#---------------------------- subroutines ---------------#

sub print_usage () {
    print "Usage:   $PROGNAME -H<Controller> -u<username> -p<password> -S<SpareController> -d<0|1>\n";
    print "Example: $PROGNAME -H10.154.2.1 -umanage -p\\!manage \n";
    print "\n";

    print "option -H = Controller IP, example: -H10.154.2,1 ";
    print "\n";

    print "option -u = username, example: -umanage";
    print "\n";

    print "option -p = password, example: -p\!manage";
    print "\n";

    print "option -d = Debugmode, shows complete output. -d0 = debug off; -d1 = debug on";
    print "\n";

    print "option -S = Spare Controller, example: -S10.154.3.1";
    print "\n";
    
}

sub print_help () {
    print "\n ---------- HELP ------------\n\n";
    print $PROGNAME."\n";
    print "\n";
    print_usage ();
    print "\n";
    print "Performs an overall check from a MSA P2000 Array + Expansions. \n";
    print "Checked Parts:\n";
    print "vdisks, disks, sas, power-supplies & fans, Network management ports";
    print "\n\n";
}

sub ping {
    my ($hostname) = @_;
    my $time = 0;
    my $success = 0;
    my $n = 2;
    my $cPing = Net::Ping->new();

    #print "Pinging $hostname $n times";
    foreach my $c (1 .. $n) {
        my ($ret, $duration, $ip) = $cPing->ping($hostname,1);
        if ($ret) {
            $success++;
            $time += $duration;
        }
    }
    if (not $success) {
        return ("2");
    } else {
        if ($success < $n) {
            #print( ($n - $success), " lost packets. Packet loss ratio: ", int ( 100 * ($n - $success) / $n ));
            return ("2");
        }
        # print "Average round trip: ", ($time / $success);
        return ("0");
    }
}











